@@ -15,6 +15,7 @@ use crate::{
 	action::{ActionRunner, ActionValidationError},
 	backend::Backend,
 	database::DatabaseError,
+	util::InnerUnwrap,
 	Database, Entry,
 };
 
@@ -169,7 +170,7 @@ impl<B: Backend> Gateway<B> {
 		let exists = self.get::<S>(&table_name)?;
 
 		if exists.is_some() {
-			return Ok(unsafe { exists.unwrap_unchecked() });
+			return Ok(unsafe { exists.inner_unwrap() });
 		}
 
 		self.create::<S>(table_name).await
@@ -223,7 +224,7 @@ impl<B: Backend> Gateway<B> {
 				return Ok(None);
 			}
 
-			temp.unwrap_unchecked()
+			temp.inner_unwrap()
 		};
 
 		map_ref.value().check::<S>()?;
@@ -276,10 +277,7 @@ impl<B: Backend> Gateway<B> {
 	{
 		let table = self.get_unchecked::<S>(table_name);
 
-		self.backend
-			.delete_table(&table.name)
-			.await
-			.unwrap_unchecked();
+		self.backend.delete_table(&table.name).await.inner_unwrap();
 
 		self.databases.remove(&table.name);
 	}
@@ -298,9 +296,9 @@ impl<B: Backend> Gateway<B> {
 	where
 		S: Entry + 'static,
 	{
-		let map_ref = self.databases.get(table_name).unwrap_unchecked();
+		let map_ref = self.databases.get(table_name).inner_unwrap();
 
-		map_ref.value().check::<S>().unwrap_unchecked();
+		map_ref.value().check::<S>().inner_unwrap();
 
 		DbRef::new(map_ref)
 	}
