@@ -1,4 +1,3 @@
-#![feature(never_type, doc_cfg, try_trait_v2)]
 #![warn(
 	clippy::pedantic,
 	clippy::nursery,
@@ -15,6 +14,7 @@
 	clippy::no_effect_underscore_binding,
 	deprecated
 )]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![cfg_attr(not(test), warn(clippy::panic_in_result_fn))]
 //! A simple database system that allows the use of multiple different backends.
 
@@ -24,6 +24,8 @@ mod database;
 mod entry;
 pub mod error;
 pub mod gateway;
+#[cfg(not(tarpaulin_include))]
+mod util;
 
 #[doc(inline)]
 pub use self::{action::Action, error::ChartError as Error, gateway::Gateway};
@@ -37,5 +39,5 @@ pub type ChartResult<T, B> = Result<T, Error<B>>;
 
 /// The helper derive macro for easily implementing [`IndexEntry`].
 #[cfg(feature = "derive")]
-#[doc(cfg(feature = "derive"))]
+#[cfg_attr(docsrs, doc(cfg(feature = "derive")))]
 pub use starchart_derive::IndexEntry;
